local L = PetSelector.L
local GetStablePetInfo = GetStablePetInfo
local GetSpellInfo = GetSpellInfo
local white = "|cffffffff"

-- Returns the currently suggested pet
-- Icon, Name, Level, Family, Type (Ferocity, Cunning, Tenacity), Stable Slot, Is it exotic?

function PetSelector:GetSelectedPet()

	local pet = self.selectedpet
	
	if pet then
	
		return pet.icon, pet.name, pet.level, pet.family, pet.class, pet.slot, pet.exotic
		
	else
	
		return nil

	end
			
end

-- Attempts to create/edit a macro so summon the selected pet

function PetSelector:SetPetMacro()

	local pet = self.selectedpet
	local mid, body, nid
	
	if not pet then return end

	body = "" --"#show Call Pet\n"
	
	--pet is in the stable
	if pet.slot > 5 then 
	
		StaticPopupDialogs["PETSELECTOR_STABLED"].text = string.format(L["%s is stabled"], pet.name)
		body = body .. "/run StaticPopup_Show(\"PETSELECTOR_STABLED\")"
		
	elseif pet.slot == 0 then
		
		--pet is already active
		--body = body .. "/cast [@pet, dead] Revive Pet\n/cast [nopet] Call Pet " .. pet.slot
		body = body .. L["revivemacro"] .. "\n" .. string.format(L["callmacro"], pet.slot)

	else
	
		--body = body .. "/cast [pet] Dismiss Pet\n/cast [nopet] Call Pet " .. pet.slot
		body = body .. L["dismissmacro"] .. "\n" .. string.format(L["callmacro"], pet.slot)

	end

	--does the macro already exist?
	mid = GetMacroIndexByName("Pet Selector")
	
	if mid == 0 then

		--no extant macro, so create one
		mid = CreateMacro("Pet Selector", 109, body)
		
	else

		--edit an existing macro
		nid = EditMacro(mid, nil, nil, body)
		
	end
	
end

--[[
	Returns a table of (all) the hunter's pets within 5 levels of the player's current level
	AllPets = include stabled pets (any non-nil value)
	returns:
 [1] = {
	icon = In game pet icon,
	name = The pet's name,
	level = The pet's level,
	family = The pet's family (localised),
	class = Tenacity | Ferocity | Cunning (localised),
	slot = Stable slot (anything over 5 is with the stable master)
	exotic = Is this an exotic pet?
 }
 --]]

function PetSelector:GetPets(AllPets)

	local pets = {}
	local icon, name, level, family, class, slot
	
	for slot = 1, 25 do
	
		icon, name, level, family, class = GetStablePetInfo(slot)
		
		if name then
		
			if self.debug then
			
				--print(slot .. ": " .. name .. " l: " .. level)
				
			end
			
			if (self.PlayerLevel - level < 4) then
			
				if ((slot > 5) and (self.db.profile.stabled or AllPets)) or (slot < 6) then
				
					table.insert(pets, {
						icon = icon, name = name, level = level, family = family, class = class, slot = slot, exotic = self:IsExotic(family)
					})
					
				end
				
			end
			
		end
		
	end

	return pets
	
end

local function tcopy(t)

	local u = {}
	local k, v
	
	for k, v in pairs(t) do u[k] = v end
	
	return setmetatable(u, getmetatable(t))

end

--[[
	Returns a table of any pet buffs the hunter cannot currently provide
	returns:
 [1] = {
	family = The pet's family (localised),
	buff = The spell ID of the buff that pet can provide
 }
 --]]
 
function PetSelector:GetMissingPets()

	local missing = tcopy(self.PetBuffs)
	local pets = self:GetPets(true)
	local detail = {}
	local pet, buff, ptype, hpet, buff, buffs, idx
	
	for idx, pet in ipairs(self.PetBuffs) do
	
		for _, ptype in ipairs(pet) do
		
			for _, hpet in ipairs(pets) do
			
				if hpet.family == ptype then
				
					missing[idx] = nil
					break

				end
				
			end
			
		end
		
	end

	if #missing > 0 then
	
		for _, pets in ipairs(missing) do
		
			for _, pet in ipairs(pets) do
			
				buff = nil
				
				for _, buffs in ipairs(self.PetInfo) do
				
					if buffs[1] == pet then
					
						buff = buffs[2]
						
					end
					
				end
				
				table.insert(detail,{family = pet, buff = buff})
				
			end
			
		end
	
	end
	
	return detail

end

function PetSelector:PopulateMissing()

	local missing = self:GetMissingPets()
	local pet, index, sicon
	
	if #missing == 0 then
	
		_G["PetSelectorPetProvider1"]:SetText(white .. L["No missing pets"])
		
	else
	
		for index, pet in ipairs(missing) do
	
			if index > 12 then break end
		
			sicon = select(3, GetSpellInfo(pet.buff))
			_G["PetSelectorPetIcon" .. index]:SetTexture(sicon)
			_G["PetSelectorPet" .. index].spellid = pet.buff
			_G["PetSelectorPetProvider" .. index]:SetText(white .. pet.family)
			
		end
		
	end
	
end

-- Returns true if petfamily is an exotic pet family
-- returns nil otherwise
-- petfamily is the localised pet family name

function PetSelector:IsExotic(petfamily)

	local exotic, family
	
	for _, family in ipairs(self.ExoticPets) do
	
		if petfamily == family then
		
			exotic = true
			break
			
		end
		
	end
	
	return exotic

end